#
# address.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'amstd/must'
require 'amstd/bug'
require 'amstd/futils'


module EST

  class AddressEntry
    
    def initialize( idx, name, addr, keys = nil )
      @index = idx
      @name = name
      @addr = addr
      self.properties = keys || []
    end

    attr :indexes

    attr :index
    attr :addr, true
    alias address addr
    attr :name, true

    def properties
      @indexes.keys
    end

    def properties=( arr )
      @indexes = h = {}
      arr.each {|k| h[k] = true }
    end

    def add_property( prop )
      @indexes[prop] = true
    end

    def remove_property( prop )
      @indexes.delete prop
    end

  end


  class AddressManager

    def initialize
      @whole = {}
      @index_table = {}
      do_initialize
    end

    
    def []( name )
      @whole[ name ]
    end

    def add( name, rc )
      idx = rc['index']
      ent = AddressEntry.new( name, rc['name'], rc['address'], idx )
      add_entry name, ent
      if idx then
        idx.each do |i|
          unless a = @index_table[i] then
            @index_table[i] = a = []
          end
          a.push ent
        end
      end
    end

    def add_entry( name, entry )
      if @whole.has_key? name then
        raise ArgumentError, "same address key '#{name}' was used"
      end
      @whole[ name ] = entry
    end

    def each
      @whole.each {|k,ent| yield ent }
    end

    def entries
      @whole.values
    end
    alias values entries

    def properties
      a = @index_table.keys
      a.sort!
      a
    end

    def new_property( prop )
      @index_table[ prop ] ||= []
    end

    def lookup_and( keys )
      ag = AddressGroup.new( @whole )
      keys.each do |k|
        ag = ag._and_( k )
      end
      ag.to_a
    end

    def lookup_or( keys )
      ret = {}
      keys.each do |k|
        if arr = @index_table[k] then
          arr.each {|a| ret[a] = true }
        end
      end
      ret.keys
    end

  end


  class AddressGroup

    def initialize( whole, grp = nil )
      @whole = whole
      @group = grp
      @i = 0
    end

    def to_a
      @group.dup
    end

    def _and_( key )
      grp = @group.find_all {|addr| addr.indexes[key] }
      type.new( @whole, grp )
    end

    def _or_( key )
      grp = {}
      @group.each do |addr|
        grp[addr] = true
      end
      @whole.each do |addr,vv|
        if addr.keys[key] then
          grp[addr] = true
        end
      end
      type.new( @whole, grp.keys )
    end

  end

end   # module EST
